/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.formsolutions;

import static de.ozgcloud.eingang.semantik.enginebased.formsolutions.FormSolutionsEngineBasedAdapter.*;
import static java.util.Objects.*;

import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

import org.springframework.stereotype.Component;

import de.ozgcloud.eingang.common.formdata.FormData;

@Component
class FormSolutionsPanelMapper implements FormSolutionsEngineBasedMapper {
	public static final String COMPONENTS = "components";
	public static final String STRING_VALUE = "stringValue";
	public static final String PANELS = "panels";

	@Override
	public FormData parseFormData(FormData formData) {
		var resultMap = mapPanels(getPanels(formData)).orElseGet(HashMap::new);
		resultMap.putAll(formData.getFormData());

		return formData.toBuilder().formData(resultMap).build();
	}

	private Optional<Map<String, Object>> mapPanels(List<Map<String, Object>> panels) {
		if (panels.isEmpty()) {
			return Optional.empty();
		}
		var resultMap = new LinkedHashMap<String, Object>();
		for (Map<String, Object> panel : panels) {
			var identifier = (String) panel.get(IDENTIFIER_KEY);
			if (isNull(identifier)) {
				continue;
			}
			getValue(panel).ifPresent(value -> resultMap.put(identifier, value));
		}

		return Optional.of(resultMap);
	}

	private Optional<Object> getValue(Map<String, Object> panel) {
		 return mapPanels(getComponentList(panel))
				 .map(Object.class::cast)
				 .or(() -> Optional.ofNullable(panel.get(STRING_VALUE)));
	}

	@SuppressWarnings("unchecked")
	public static List<Map<String, Object>> getPanels(FormData formData) {
		if (isNull(formData)) {
			return List.of();
		}
		return Optional.ofNullable(formData.getFormData())
				.map(formDataMap -> (Map<String, Object>) formDataMap.get(ASSISTANT))
				.map(assistent -> (List<Map<String, Object>>) assistent.get(PANELS))
				.orElse(List.of());
	}

	@SuppressWarnings("unchecked")
	public static List<Map<String, Object>> getComponentList(Map<String, Object> panel) {
		if (isNull(panel)) {
			return List.of();
		}
		return Optional.ofNullable(panel.get(COMPONENTS)).map(c -> (List<Map<String, Object>>) c).orElse(List.of());
	}

}
